@@ -16,10 +16,13 @@
 
 import abc
 import collections
-import six
 
+from oslo.utils import netutils
+import six
 from stevedore import driver
 
+import tooz
+
 TOOZ_BACKENDS_NAMESPACE = "tooz.backends"
 
 
@@ -60,6 +63,7 @@ class LeaderElected(Event):
 class CoordinationDriver(object):
 
     def __init__(self):
+        self._started = False
         self._hooks_join_group = collections.defaultdict(Hooks)
         self._hooks_leave_group = collections.defaultdict(Hooks)
         self._hooks_elected_leader = collections.defaultdict(Hooks)
@@ -70,9 +74,10 @@ class CoordinationDriver(object):
         return (len(self._hooks_join_group[group_id])
                 + len(self._hooks_leave_group[group_id]))
 
-    @abc.abstractmethod
-    def run_watchers(self):
+    @staticmethod
+    def run_watchers():
         """Run the watchers callback."""
+        raise tooz.NotImplemented
 
     @abc.abstractmethod
     def watch_join_group(self, group_id, callback):
@@ -163,28 +168,39 @@ class CoordinationDriver(object):
 
         :param group_id: The group where we don't want to be a leader anymore
         """
-        raise NotImplementedError
+        raise tooz.NotImplemented
 
-    def start(self, timeout):
+    def start(self):
         """Start the service engine.
 
         If needed, the establishment of a connection to the servers
         is initiated.
-
-        :param timeout: Time in seconds to wait for connection to succeed.
-        :type timeout: int
         """
+        if self._started:
+            raise ToozError(
+                "Can not start a driver which has not been stopped")
+        self._start()
+        self._started = True
+
+    def _start(self):
+        pass
 
-    @abc.abstractmethod
     def stop(self):
         """Stop the service engine.
 
         If needed, the connection to servers is closed and the client will
         disappear from all joined groups.
         """
+        if not self._started:
+            raise ToozError("Can not stop a driver which has not been started")
+        self._stop()
+        self._started = False
 
-    @abc.abstractmethod
-    def create_group(self, group_id):
+    def _stop(self):
+        pass
+
+    @staticmethod
+    def create_group(group_id):
         """Request the creation of a group asynchronously.
 
         :param group_id: the id of the group to create
@@ -192,17 +208,19 @@ class CoordinationDriver(object):
         :returns: None
         :rtype: CoordAsyncResult
         """
+        raise tooz.NotImplemented
 
-    @abc.abstractmethod
-    def get_groups(self):
+    @staticmethod
+    def get_groups():
         """Return the list composed by all groups ids asynchronously.
 
         :returns: the list of all created group ids
         :rtype: CoordAsyncResult
         """
+        raise tooz.NotImplemented
 
-    @abc.abstractmethod
-    def join_group(self, group_id, capabilities=b""):
+    @staticmethod
+    def join_group(group_id, capabilities=b""):
         """Join a group and establish group membership asynchronously.
 
         :param group_id: the id of the group to join
@@ -212,9 +230,10 @@ class CoordinationDriver(object):
         :returns: None
         :rtype: CoordAsyncResult
         """
+        raise tooz.NotImplemented
 
-    @abc.abstractmethod
-    def leave_group(self, group_id):
+    @staticmethod
+    def leave_group(group_id):
         """Leave a group asynchronously.
 
         :param group_id: the id of the group to leave
@@ -222,18 +241,20 @@ class CoordinationDriver(object):
         :returns: None
         :rtype: CoordAsyncResult
         """
+        raise tooz.NotImplemented
 
-    @abc.abstractmethod
-    def get_members(self, group_id):
+    @staticmethod
+    def get_members(group_id):
         """Return the list of all members ids of the specified group
         asynchronously.
 
         :returns: list of all created group ids
         :rtype: CoordAsyncResult
         """
+        raise tooz.NotImplemented
 
-    @abc.abstractmethod
-    def get_member_capabilities(self, group_id, member_id):
+    @staticmethod
+    def get_member_capabilities(group_id, member_id):
         """Return the capabilities of a member asynchronously.
 
         :param group_id: the id of the group of the member
@@ -243,9 +264,10 @@ class CoordinationDriver(object):
         :returns: capabilities of a member
         :rtype: CoordAsyncResult
         """
+        raise tooz.NotImplemented
 
-    @abc.abstractmethod
-    def update_capabilities(self, group_id, capabilities):
+    @staticmethod
+    def update_capabilities(group_id, capabilities):
         """Update capabilities of the caller in the specified group
         asynchronously.
 
@@ -256,6 +278,7 @@ class CoordinationDriver(object):
         :returns: None
         :rtype: CoordAsyncResult
         """
+        raise tooz.NotImplemented
 
     @staticmethod
     def get_leader(group_id):
@@ -265,7 +288,7 @@ class CoordinationDriver(object):
         :returns: the leader
         :rtype: CoordAsyncResult
         """
-        raise NotImplementedError
+        raise tooz.NotImplemented
 
     @staticmethod
     def get_lock(name):
@@ -275,13 +298,14 @@ class CoordinationDriver(object):
                      nodes.
 
         """
-        raise NotImplementedError
+        raise tooz.NotImplemented
 
     @staticmethod
     def heartbeat():
         """Method to run once in a while to be sure that the member is not dead
         and is still an active member of a group.
 
+
         """
         pass
 
@@ -305,23 +329,21 @@ class CoordAsyncResult(object):
         """Returns True if the task is done, False otherwise."""
 
 
-    #TODO(yassine)
-    #Replace kwargs by something more simple.
-def get_coordinator(backend, member_id, **kwargs):
+def get_coordinator(backend_url, member_id):
     """Initialize and load the backend.
 
-    :param backend: the current tooz provided backends are 'zookeeper'
+    :param backend_url: the backend URL to use
     :type backend: str
     :param member_id: the id of the member
     :type member_id: str
-    :param kwargs: additional backend specific options
-    :type kwargs: dict
     """
-    return driver.DriverManager(namespace=TOOZ_BACKENDS_NAMESPACE,
-                                name=backend,
-                                invoke_on_load=True,
-                                invoke_args=(member_id,),
-                                invoke_kwds=kwargs).driver
+    parsed_url = netutils.urlsplit(backend_url)
+    parsed_qs = six.moves.urllib.parse.parse_qs(parsed_url.query)
+    return driver.DriverManager(
+        namespace=TOOZ_BACKENDS_NAMESPACE,
+        name=parsed_url.scheme,
+        invoke_on_load=True,
+        invoke_args=(member_id, parsed_url, parsed_qs)).driver
 
 
 class ToozError(Exception):
@@ -336,6 +358,10 @@ class ToozConnectionError(ToozError):
     """
 
 
+class OperationTimedOut(ToozError):
+    """Exception raised when an operation times out."""
+
+
 class GroupNotCreated(ToozError):
     """Exception raised when the caller request a group which does
     not exist.
