@@ -17,6 +17,7 @@
 # under the License.
 
 import hashlib
+import time
 
 import six
 import sysv_ipc
@@ -68,10 +69,13 @@ class IPCLock(locking.Lock):
         # Convert blocking argument to a valid timeout value
         if blocking is True:
             timeout = None
+            start_time = None
         elif blocking is False:
             timeout = 0
+            start_time = None
         else:
             timeout = blocking
+            start_time = time.time()
         while True:
             try:
                 self._lock = sysv_ipc.Semaphore(self.key,
@@ -88,9 +92,18 @@ class IPCLock(locking.Lock):
                     # Semaphore has been deleted in the mean time, retry from
                     # the beginning!
                     continue
+            if start_time is not None:
+                elapsed = max(0.0, time.time() - start_time)
+                if elapsed >= timeout:
+                    # Ran out of time...
+                    return False
+                adjusted_timeout = timeout - elapsed
+            else:
+                adjusted_timeout = timeout
             try:
-                self._lock.acquire(timeout=timeout)
+                self._lock.acquire(timeout=adjusted_timeout)
             except sysv_ipc.BusyError:
+                self._lock = None
                 return False
             except sysv_ipc.ExistentialError:
                 # Likely the lock has been deleted in the meantime, retry
