@@ -13,6 +13,7 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+import os
 import time
 import uuid
 
@@ -28,11 +29,25 @@ class TestAPI(testscenarios.TestWithScenarios,
               tests.TestCaseSkipNotImplemented):
 
     scenarios = [
-        ('zookeeper', {'url': 'kazoo://127.0.0.1:2181?timeout=5'}),
+        ('zookeeper', {'url': 'kazoo://127.0.0.1:2181?timeout=5',
+                       'bad_url': 'kazoo://localhost:1'}),
         ('zake', {'url': 'zake://?timeout=5'}),
-        ('memcached', {'url': 'memcached://?timeout=5'}),
+        ('memcached', {'url': 'memcached://?timeout=5',
+                       'bad_url': 'memcached://localhost:1'}),
         ('ipc', {'url': 'ipc://'}),
-        ('redis', {'url': 'redis://localhost:6379?timeout=5'}),
+        ('redis', {'url': 'redis://localhost:6379?timeout=5',
+                   'bad_url': 'redis://localhost:1'}),
+        ('postgresql', {'url': os.getenv("TOOZ_TEST_PGSQL_URL"),
+                        'bad_url': 'postgresql://localhost:1'}),
+        ('mysql', {'url': os.getenv("TOOZ_TEST_MYSQL_URL"),
+                   'bad_url': 'mysql://localhost:1'}),
+    ]
+
+    # Only certain drivers have the tested support for timeouts that we test
+    # here, these are the lists of driver types that do support our test type.
+    timeout_capable = [
+        'memcached://',
+        'redis://',
     ]
 
     def assertRaisesAny(self, exc_classes, callable_obj, *args, **kwargs):
@@ -42,8 +57,20 @@ class TestAPI(testscenarios.TestWithScenarios,
         callable_obj = testcase.Nullary(callable_obj, *args, **kwargs)
         self.assertThat(callable_obj, matcher)
 
+    def skipIfNotSupported(self, supported):
+        applicable = False
+        for prefix in supported:
+            if self.url.startswith(prefix):
+                applicable = True
+                break
+        if not applicable:
+            self.skipTest("This test only works with %s types for now"
+                          % list(supported))
+
     def setUp(self):
         super(TestAPI, self).setUp()
+        if self.url is None:
+            self.skipTest("No URL set for this driver")
         self.group_id = self._get_random_uuid()
         self.member_id = self._get_random_uuid()
         self._coord = tooz.coordination.get_coordinator(self.url,
@@ -57,6 +84,14 @@ class TestAPI(testscenarios.TestWithScenarios,
         self._coord.stop()
         super(TestAPI, self).tearDown()
 
+    def test_connection_error(self):
+        if not hasattr(self, "bad_url"):
+            raise testcase.TestSkipped("No bad URL provided")
+        coord = tooz.coordination.get_coordinator(self.bad_url,
+                                                  self.member_id)
+        self.assertRaises(tooz.coordination.ToozConnectionError,
+                          coord.start)
+
     def test_stop_first(self):
         c = tooz.coordination.get_coordinator(self.url,
                                               self.member_id)
@@ -214,8 +249,7 @@ class TestAPI(testscenarios.TestWithScenarios,
         self.assertTrue(member_id_test2 not in members_ids)
 
     def test_timeout(self):
-        if not self.url.startswith('memcached://'):
-            self.skipTest("This test only works with memcached for now")
+        self.skipIfNotSupported(self.timeout_capable)
         member_id_test2 = self._get_random_uuid()
         client2 = tooz.coordination.get_coordinator(self.url,
                                                     member_id_test2)
@@ -314,6 +348,38 @@ class TestAPI(testscenarios.TestWithScenarios,
         self._coord.run_watchers()
         self.assertIsNone(self.event)
 
+    def test_watch_join_group_disappear(self):
+        if not hasattr(self._coord, '_destroy_group'):
+            self.skipTest("This test only works with coordinators"
+                          " that have the ability to destroy groups.")
+
+        self._coord.create_group(self.group_id).get()
+        self._coord.watch_join_group(self.group_id, self._set_event)
+        self._coord.watch_leave_group(self.group_id, self._set_event)
+
+        member_id_test2 = self._get_random_uuid()
+        client2 = tooz.coordination.get_coordinator(self.url,
+                                                    member_id_test2)
+        client2.start()
+        client2.join_group(self.group_id).get()
+
+        while True:
+            if self._coord.run_watchers():
+                break
+        self.assertIsInstance(self.event,
+                              tooz.coordination.MemberJoinedGroup)
+        self.event = None
+
+        # Force the group to disappear...
+        self._coord._destroy_group(self.group_id)
+
+        while True:
+            if self._coord.run_watchers():
+                break
+
+        self.assertIsInstance(self.event,
+                              tooz.coordination.MemberLeftGroup)
+
     def test_watch_join_group_non_existent(self):
         self.assertRaises(tooz.coordination.GroupNotCreated,
                           self._coord.watch_join_group,
@@ -321,6 +387,34 @@ class TestAPI(testscenarios.TestWithScenarios,
                           lambda: None)
         self.assertEqual(0, len(self._coord._hooks_join_group[self.group_id]))
 
+    def test_watch_join_group_booted_out(self):
+        self._coord.create_group(self.group_id).get()
+        self._coord.join_group(self.group_id).get()
+        self._coord.watch_join_group(self.group_id, self._set_event)
+        self._coord.watch_leave_group(self.group_id, self._set_event)
+
+        member_id_test2 = self._get_random_uuid()
+        client2 = tooz.coordination.get_coordinator(self.url,
+                                                    member_id_test2)
+        client2.start()
+        client2.join_group(self.group_id).get()
+
+        while True:
+            if self._coord.run_watchers():
+                break
+
+        client3 = tooz.coordination.get_coordinator(self.url, self.member_id)
+        client3.start()
+        client3.leave_group(self.group_id)
+
+        # Only works for clients that have access to the groups they are part
+        # of, to ensure that after we got booted out by client3 that this
+        # client now no longer believes its part of the group.
+        if hasattr(self._coord, '_joined_groups'):
+            self.assertIn(self.group_id, self._coord._joined_groups)
+            self._coord.run_watchers()
+            self.assertNotIn(self.group_id, self._coord._joined_groups)
+
     def test_watch_leave_group_non_existent(self):
         self.assertRaises(tooz.coordination.GroupNotCreated,
                           self._coord.watch_leave_group,
